# EZFilter release 1 by David Bowland
# ./addons/eventscripts/ezfilter/ezfilter.py

# >>> To configure this addon please see ezfilter.cfg <<<

"""
Filters words or spam from chat. Requires only ES 2.0+
"""


import es
import gamethread
import langlib
import os.path
import playerlib
import random


info = es.AddonInfo()
info.name = 'EZFilter'
info.version = '1'
info.url = 'http://addons.eventscripts.com/addons/view/ezfilter'
info.basename = 'ezfilter'
info.author = 'SuperDave'


dict_filtered_words = {}
dict_players = {}
dict_player_template = {'filter_count':0, 'mute':0, 'immune':0, 'last_line':0, 'spam':0}
dict_say_commands = {}

dict_events = {'player_filtered':{'userid':'short', 'strike':'short', 'reason':'string'}}
dict_options = {'ezfilter_replace_characters':['@#$%&*', 'Characters to use when masking a filtered words without a replacement word'], 'ezfilter_replace_line':[0, '0 = replace or mask filtered words, 1 = suppress chat that contains filtered words'], 'ezfilter_replace_strikes':[1,' 0 = filtering a word adds no strikes, 1 = filtering any number of words in a line containing is considered a strike, 2 = every word that is filtered is considered a strike'], 'ezfilter_replace_names':[1, '0 = no change, 1 = mask or replace filtered words in names when players connect but kick players who retain a filtered name, 2 = kick all players with a filtered word in their name'], 'ezfilter_spam_delay':[2 ,'Number of seconds after saying a line players must wait to say the next line--set to 0 to remove'], 'ezfilter_spam_duplicate':[1, '0 = no change, 1 = chat that is a duplicate of the player\'s previous chat is considered spam'], 'ezfilter_spam_strikes':[1, '0 = filtering spam adds no strikes, 1 = filtering a line as spam is considered a strike'], 'ezfilter_punishment_wordcount':[0, '0 = each line containing a filtered word is considered a strike, 1 = every word that is filtered is considered a strike'], 'ezfilter_punishment_strikes_clear':[2, '0 = never clear stikes, 1 = clear strikes every time EZFilters is loaded, 2 = clear strikes every map'], 'ezfilter_punishment_warn_strikes':[3, 'Number of strikes a player must have to be warned--set to 0 to remove this punishment'], 'ezfilter_punishment_mute_strikes':[5, 'Number of strikes a player must have to be muted--set to 0 to remove this punishment'], 'ezfilter_punishment_mute_delay':[60, 'Number of seconds to mute players--set to 0 for effect to remain while EZFilter is loaded'], 'ezfilter_punishment_kick_strikes':[8, 'Number of strikes a player must have to be kicked--set to 0 to remove this punishment'], 'ezfilter_punishment_ban_strikes':[10, 'Number of strikes a player must have to be banned--set to 0 to remove this punishment'], 'ezfilter_punishment_ban_time':[0, 'Minutes to ban players--set to 0 for permanent'], 'ezfilter_announce_warn_chat':[1, '0 = no change, 1 = players receive the warn punishment in chat area'], 'ezfilter_announce_warn_center':[1, '0 = no change, 1 = players receive the warn punishment in center text'], 'ezfilter_announce_warn_hud':[1, '0 = no change, 1 = players receive the warn punishment in a HUD message'], 'ezfilter_announce_mute_console':[0, '0 = no change, 1 = announce to the player on the console when he or she is muted, 2 = announce to all players on the console when a player is muted'], 'ezfilter_announce_mute_chat':[2, '0 = no change, 1 = announce to the player in chat area when he or she is muted, 2 = announce to all players in chat area when a player is muted'], 'ezfilter_announce_mute_center':[1, '0 = no change, 1 = announce to the player in center text when he or she is muted'], 'ezfilter_announce_mute_hud':[1, '0 = no change, 1 = announce to the player in a HUD message when he or she is muted'], 'ezfilter_announce_unmute_console':[0, '0 = no change, 1 = announce to the player on the console when he or she is unmuted, 2 = announce to all players on the console when a player is unmuted'], 'ezfilter_announce_unmute_chat':[2, '0 = no change, 1 = announce to the player in chat area when he or she is unmuted, 2 = announce to all players in chat area when a player is unmuted'], 'ezfilter_announce_unmute_center':[1, '0 = no change, 1 = announce to the player in center text when he or she is unmuted'], 'ezfilter_announce_unmute_hud':[1, '0 = no change, 1 = announce to the player in a HUD message when he or she is unmuted'], 'ezfilter_announce_ban_console':[0, '0 = no change, 1 = announce to all players on the console when a player is banned'], 'ezfilter_announce_ban_chat':[1, '0 = no change, 1 = announce to all players in chat area when a player is banned']}
func_lang_text = None


def load():
   """
   Checks to make sure critical server variables are created by the config
   Calls load_events
   Loads the database of strikes if necessary
   Calls player_spawn for each player
   Registers the say filter
   """
   global dict_players
   global func_lang_text

   if not es.exists('command', 'ezfilter_add'):
      es.regcmd('ezfilter_add', 'ezfilter/filter_cmd', 'ezfilter_add <\"word\"> [\"replacement word\"]\nServer command for adding say filters')
   if not es.exists('command', 'ezfilter_remove'):
      es.regcmd('ezfilter_remove', 'ezfilter/filter_cmd', 'ezfilter_remove <\"word\">\nServer command for removing say filters')
   if not es.exists('command', 'ezfilter_command_add'):
      es.regcmd('ezfilter_command_add', 'ezfilter/say_command_cmd', 'ezfilter_command_add <command> [0 = do not filter words after command, 1 = filter words after command]\nServer command for designating a word as a say command')
   if not es.exists('command', 'ezfilter_command_remove'):
      es.regcmd('ezfilter_command_remove', 'ezfilter/say_command_cmd', 'ezfilter_command_remove <command>\nServer command for removing a word designated as a say command')
   if not es.exists('command', 'ezfilter_immunity_add'):
      es.regcmd('ezfilter_immunity_add', 'ezfilter/immunity_cmd', 'ezfilter_immunity_add <\"Steam ID\"> [0 = immune from punishments, 1 = immune from punishments and filters]\nServer command for designating players as immune')
   if not es.exists('command', 'ezfilter_immunity_remove'):
      es.regcmd('ezfilter_immunity_remove', 'ezfilter/immunity_cmd', 'ezfilter_immunity_remove <\"Steam ID\">\nServer command for removing immune players')
   if not es.exists('command', 'ezfilter_mute'):
      es.regcmd('ezfilter_mute', 'ezfilter/mute_cmd', 'ezfilter_mute <userid/Steam ID/player name> [duration in seconds--set to 0 for effect to remain while EZFilter is loaded]\nServer command for muting players')

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('ezfilter') + '/ezfilter.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/ezfilter/ezfilter.cfg')
   else:
      es.dbgmsg(0, 'EZFilter: Unable to load ezfilter.cfg! Please ensure it is in the ./ezfilter/ directory.')

   str_lang_path = es.getAddonPath('ezfilter') + '/ezfilter_languages.ini'
   if os.path.isfile(str_lang_path):
      func_lang_text = langlib.Strings(str_lang_path)
   else:
      es.dbgmsg(0, 'EZFilter: Unable to load ezfilter_languages.ini! Please ensure it is in the ./ezfilter/ directory.')
   if not func_lang_text:
      func_lang_text = missing_lang

   load_events()

   if not int(es.ServerVar('ezfilter_punishment_strikes_clear')):
      str_file_path = es.getAddonPath('ezfilter') + '/ezfilter_strikes.db'
      if os.path.isfile(str_file_path):
         file_database = open(str_file_path)
         list_file_input = file_database.readlines()
         file_database.close()

         for str_line in list_file_input:
            if '=' in str_line:
               list_items = str_line.split('=')
               str_steamid = list_items[0]
               if not dict_players.has_key(str_steamid):
                  dict_players[str_steamid] = dict_player_template.copy()
               dict_players[str_steamid]['filter_count'] = int(list_items[1])

   for int_userid in es.getUseridList():
      player_spawn({'userid':str(int_userid), 'es_username':es.getplayername(int_userid), 'es_steamid':es.getplayersteamid(int_userid)})

   es.addons.registerSayFilter(say_filter)


def es_map_start(event_var):
   """
   Calls load_events
   Clears the strikes from players if necessary
   Calls save_strikes
   """
   global dict_players

   load_events()

   if int(es.ServerVar('ezfilter_punishment_strikes_clear')) == 2:
      for str_steamid in dict_players:
         dict_players[str_steamid]['filter_count'] = 0

   save_strikes()


def round_start(event_var):
   """Kicks players with inappropriate names"""
   if int(es.ServerVar('ezfilter_replace_names')):
      for int_userid in es.getUseridList():
         int_filter_count, str_name = filter_string(es.getplayername(int_userid))
         str_steamid = es.getplayersteamid(int_userid)
         if dict_players.has_key(str_steamid) and int_filter_count:
            if not dict_players[str_steamid]['immune']:
               es.server.cmd('kickid %s %s' % (int_userid, func_lang_text('kick name', {}, playerlib.getPlayer(int_userid).get('lang'))))


def player_spawn(event_var):
   """Renames or kicks the player if the name is inappropriate"""

   int_option = int(es.ServerVar('ezfilter_replace_names'))
   if int_option:
      int_filter_count, str_name = filter_string(event_var['es_username'])
      if int_filter_count:
         str_userid = event_var['userid']

         str_steamid = event_var['es_steamid']
         if dict_players.has_key(str_steamid):
            int_immune = dict_players[str_steamid]['immune']
         else:
            int_immune = 0
         if int_option == 2 and not int_immune:
            es.server.cmd('kickid %s %s' % (str_userid, func_lang_text('kick name', {}, playerlib.getPlayer(str_userid).get('lang'))))
         elif int_immune <> 2:
            es.cexec(str_userid, 'name \"%s\"' % str_name)


def player_changename(event_var):
   """Kicks the player if the name is inappropriate"""

   int_option = int(es.ServerVar('ezfilter_replace_names'))
   if int_option:
      str_steamid = event_var['es_steamid']
      if dict_players.has_key(str_steamid):
         int_immune = dict_players[str_steamid]['immune']
      else:
         int_immune = 0
      if not int_immune:
         int_filter_count, str_name = filter_string(event_var['newname'])
         if int_filter_count:
            str_userid = event_var['userid']
            es.server.cmd('kickid %s %s' % (str_userid, func_lang_text('kick name', {}, playerlib.getPlayer(str_userid).get('lang'))))


def unload():
   """
   Unregisters the say filter
   Calls save_strikes
   Cancels outstanding delays
   """
   es.addons.unregisterSayFilter(say_filter)

   save_strikes()

   for str_steamid in dict_players:
      dict_current_player = dict_players[str_steamid]
      if dict_current_player['mute'] == 1:
         gamethread.cancelDelayed('ezfilter_mute_%s' % str_steamid)
      if dict_current_player['spam']:
         gamethread.cancelDelayed('ezfilter_spam_%s' % str_steamid)

   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def say_filter(int_userid, str_message, int_team_only):
   """
   Receives each player's userid, message, and team only flag and returns the userid, filtered message, and team only flag
   Returns the unfiltered message if the player is immune
   Returns the unfiltered message if the first word was a say command
   Returns no message and fires player_filtered if the player is muted
   Returns no message and fires player_filtered if the message is spam
   Filters the message
   Punishes the player and fires player_filterd if necessary
   Starts the spam delay
   Returns the filtered message
   """
   global dict_players

   str_steamid = es.getplayersteamid(int_userid)
   if not dict_players.has_key(str_steamid):
      dict_players[str_steamid] = dict_player_template.copy()
   dict_current_players = dict_players[str_steamid]

   if dict_current_players['immune'] == 2:
      dict_current_players['last_line'] = str_message
      return int_userid, str_message, int_team_only

   int_first_space = str_message.find(' ')
   if int_first_space == -1:
      str_first_word = str_message
   else:
      str_first_word = str_message[0:int_first_space]
   if dict_say_commands.has_key(str_first_word) or es.exists('saycommand', str_first_word):
      if dict_say_commands.has_key(str_first_word):
         if not dict_say_commands[str_first_word]:
            return int_userid, str_message, int_team_only
      str_message = str_message[int_first_space:]
   else:
      str_first_word = ''

   if dict_current_players['mute'] and not str_first_word:
      es.event('initialize', 'player_filtered')
      es.event('setint', 'player_filtered', 'userid', int_userid)
      es.event('setint', 'player_filtered', 'strike', dict_current_players['filter_count'])
      es.event('setstring', 'player_filtered', 'reason', 'MUTE')
      es.event('fire', 'player_filtered')
      return 0, '', int_team_only

   if not str_first_word:
      if ((str_message == dict_current_players['last_line'] and int(es.ServerVar('ezfilter_spam_duplicate'))) or dict_current_players['spam']):
         if int(es.ServerVar('ezfilter_spam_strikes')) and not dict_current_players['immune']:
            dict_current_players['filter_count'] += 1
            filter_punish(int_userid, str_steamid, dict_current_players['filter_count'])

         es.event('initialize', 'player_filtered')
         es.event('setint', 'player_filtered', 'userid', int_userid)
         es.event('setint', 'player_filtered', 'strike', dict_current_players['filter_count'])
         es.event('setstring', 'player_filtered', 'reason', 'SPAM')
         es.event('fire', 'player_filtered')
         return 0, '', int_team_only
      dict_current_players['last_line'] = str_message

   int_filter_count, str_message = filter_string(str_message)

   if int_filter_count:
      int_strike_option = int(es.ServerVar('ezfilter_replace_strikes'))
      if int_strike_option and not dict_current_players['immune']:
         if int_strike_option == 2:
            dict_current_players['filter_count'] += int_filter_count
         else:
            dict_current_players['filter_count'] += 1

         filter_punish(int_userid, str_steamid, dict_current_players['filter_count'])

      es.event('initialize', 'player_filtered')
      es.event('setint', 'player_filtered', 'userid', int_userid)
      es.event('setint', 'player_filtered', 'strike', dict_current_players['filter_count'])
      es.event('setstring', 'player_filtered', 'reason', 'LANGUAGE')
      es.event('fire', 'player_filtered')

      if int(es.ServerVar('ezfilter_replace_line')) and not str_first_word:
         return 0, '', int_team_only

   float_spam_delay = float(es.ServerVar('ezfilter_spam_delay'))
   if float_spam_delay and not str_first_word:
      if dict_current_players['spam']:
         gamethread.cancelDelayed('ezfilter_spam_%s' % str_steamid)
      dict_current_players['spam'] = 1
      gamethread.delayedname(float_spam_delay, 'ezfilter_spam_%s' % str_steamid, remove_spam, str_steamid)

   str_message = str_first_word + str_message
   return int_userid, str_message, int_team_only


def filter_string(str_message):
   """
   Filters the string and returns the numebr of replaced words, the filtered string
   Counts the number of words replaced
   Randomizes a replacement string
   Replaces all instances of the filtered word in the string
   """

   int_filter_count = 0
   for str_word in dict_filtered_words:
      str_message_lower = str_message.lower()
      int_word_count = str_message_lower.count(str_word)
      if int_word_count:
         int_filter_count += int_word_count

         str_filter_replace = dict_filtered_words[str_word]
         if not str_filter_replace:
            for int_x in range(0, len(str_word)):
               str_filter_replace += random.choice(str(es.ServerVar('ezfilter_replace_characters')))

         int_index = str_message_lower.find(str_word)
         int_word_length = len(str_word)
         int_replace_length = len(str_filter_replace)
         while int_index >= 0:
            str_temp_filter = ''
            str_original_word = str_message[int_index:int_index + int_word_length]
            if str_original_word.isupper():
               str_temp_filter = str_filter_replace.upper()
            else:
               for int_x in range(0, int_word_length):
                  if str_original_word[int_x:int_x + 1].isupper():
                     str_temp_filter += str_filter_replace[int_x:int_x + 1].upper()
                  else:
                     str_temp_filter += str_filter_replace[int_x:int_x + 1]
               str_temp_filter += str_filter_replace[int_word_length:]
            str_message = str_message.replace(str_original_word, str_temp_filter)
            int_index = str_message.lower().find(str_word)

   return int_filter_count, str_message


def remove_spam(str_steamid):
   """Removes the spam status from the player"""
   global dict_players

   dict_players[str_steamid]['spam'] = 0


def filter_punish(int_userid, str_steamid, int_filter_count):
   """
   Punishes the player based on strikes
   Ban, kick, mute, warn
   """
   int_option_ban = int(es.ServerVar('ezfilter_punishment_ban_strikes'))
   int_option_kick = int(es.ServerVar('ezfilter_punishment_kick_strikes'))
   int_option_mute = int(es.ServerVar('ezfilter_punishment_mute_strikes'))
   int_option_warn = int(es.ServerVar('ezfilter_punishment_warn_strikes'))
   str_player_lang = playerlib.getPlayer(int_userid).get('lang')

   if int_option_ban and int_filter_count >= int_option_ban:
      es.server.cmd('banid %s %s kick' % (es.ServerVar('ezfilter_punishment_ban_time'), str_steamid))
      int_announce_console = int(es.ServerVar('ezfilter_announce_ban_console'))
      int_announce_chat = int(es.ServerVar('ezfilter_announce_ban_chat'))
      if int_announce_console or int_announce_chat:
         dict_tokens = {'player':es.getplayername(int_userid)}
         for int_loop_userid in es.getUseridList():
            if int_userid <> int_loop_userid:
               str_loop_lang = playerlib.getPlayer(int_loop_userid).get('lang')
               if int_announce_console:
                  es.cexec(int_loop_userid, 'echo ' + remove_tags(func_lang_text('ban all', dict_tokens, str_loop_lang)))
               if int_announce_chat:
                  es.tell(int_loop_userid, '#multi', func_lang_text('ban all', dict_tokens, str_loop_lang))

   elif int_option_kick and int_filter_count >= int_option_kick:
      es.server.cmd('kickid %s %s' % (str_steamid, func_lang_text('kick', {}, str_player_lang)))

   elif int_option_mute and int_filter_count >= int_option_mute:
      mute_player(int_userid, str_steamid, float(es.ServerVar('ezfilter_punishment_mute_delay')), str_player_lang)

   elif int_option_warn and int_filter_count >= int_option_warn:
      if int(es.ServerVar('ezfilter_announce_warn_chat')):
         es.tell(int_userid, '#multi', func_lang_text('warn', {}, str_player_lang))
      if int(es.ServerVar('ezfilter_announce_warn_center')):
         es.centertell(int_userid, remove_tags(func_lang_text('warn', {}, str_player_lang)))
      if int(es.ServerVar('ezfilter_announce_warn_chat')):
         es.usermsg('create', 'ezfilter_hud', 'HintText')
         es.usermsg('write', 'short', 'ezfilter_hud', -1)
         es.usermsg('write', 'string', 'ezfilter_hud', remove_tags(func_lang_text('warn', {}, str_player_lang)))
         es.usermsg('send', 'ezfilter_hud', int_userid)
         es.usermsg('delete', 'ezfilter_hud')


def mute_player(int_userid, str_steamid, float_delay, str_player_lang):
   """Designates the player as muted and starts a delay to call unmute_player"""
   global dict_players

   dict_current_player = dict_players[str_steamid]
   if dict_current_player['mute'] == 1:
      gamethread.cancelDelayed('ezfilter_mute_%s' % str_steamid)

   mute_announce('mute', int_userid, str_player_lang)

   if float_delay:
      dict_current_player['mute'] = 1
      gamethread.delayedname(float_delay, 'ezfilter_mute_%s' % str_steamid, unmute_player, str_steamid)
   else:
      dict_current_player['mute'] = 2


def unmute_player(str_steamid):
   """Removes the muted status from the player"""
   global dict_players

   dict_current_player = dict_players[str_steamid]
   dict_current_player['mute'] = 0

   int_userid = es.getuserid(str_steamid)
   if int_userid:
      mute_announce('unmute', int_userid, playerlib.getPlayer(int_userid).get('lang'))


def mute_announce(str_action, int_userid, str_player_lang):
   """Announces to a player when he or she is muted or unmuted"""
   dict_tokens = {'player':es.getplayername(int_userid)}

   int_option = int(es.ServerVar('ezfilter_announce_' + str_action + '_console'))
   if int_option:
      es.cexec(int_userid, 'echo ' + remove_tags(func_lang_text(str_action, {}, str_player_lang)))
      if int_option == 2:
         for int_loop_userid in es.getUseridList():
            if int_userid <> int_loop_userid:
               es.cexec(int_loop_userid, 'echo ' + remove_tags(func_lang_text(str_action + ' all', dict_tokens, playerlib.getPlayer(int_loop_userid).get('lang'))))
   int_option = int(es.ServerVar('ezfilter_announce_' + str_action + '_chat'))
   if int_option:
      es.cexec(int_userid, 'echo ' + remove_tags(func_lang_text(str_action, {}, str_player_lang)))
      if int_option == 2:
         for int_loop_userid in es.getUseridList():
            if int_userid <> int_loop_userid:
               es.cexec(int_loop_userid, 'echo ' + remove_tags(func_lang_text(str_action + ' all', dict_tokens, playerlib.getPlayer(int_loop_userid).get('lang'))))
   if int(es.ServerVar('ezfilter_announce_' + str_action + '_center')):
      es.centertell(int_userid, remove_tags(func_lang_text(str_action, {}, str_player_lang)))
   if int(es.ServerVar('ezfilter_announce_' + str_action + '_hud')):
      es.usermsg('create', 'ezfilter_hud', 'HintText')
      es.usermsg('write', 'short', 'ezfilter_hud', -1)
      es.usermsg('write', 'string', 'ezfilter_hud', remove_tags(func_lang_text(str_action, {}, str_player_lang)))
      es.usermsg('send', 'ezfilter_hud', int_userid)
      es.usermsg('delete', 'ezfilter_hud')


def mute_cmd():
   """
   ezfilter_mute <userid/Steam ID/player name> [duration in seconds--set to 0 for effect to remain while EZFilter is loaded]
   Mutes the player for the specified time
   """
   int_arg_count = es.getargc()
   if int_arg_count in (2, 3):
      str_arg = es.getargv(1)
      int_userid = es.getuserid(str_arg)
      if int_userid:
         if int_arg_count == 3:
            float_delay = float(es.getargv(2))
         else:
            float_delay = float(es.ServerVar('ezfilter_punishment_mute_delay'))

         mute_player(int_userid, es.getplayersteamid(int_userid), float_delay, playerlib.getPlayer(int_userid).get('lang'))
      else:
         es.dbgmsg(0, 'EZFilter: Unable to find player \"%s\"' % str_arg)
   else:
      es.dbgmsg(0, 'Syntax: ezfilter_mute <userid/Steam ID/player name> [duration in seconds--set to 0 for effect to remain while EZFilter is loaded]')


def filter_cmd():
   """
   ezfilter_add <"word"> ["replacement word"]
   ezfilter_remove <word>
   Adds or removes words that are filtered
   """
   global dict_filtered_words

   str_command = es.getargv(0).lower()
   int_arg_count = es.getargc()
   if int_arg_count in {'ezfilter_add':(2, 3), 'ezfilter_remove':(2,)}[str_command]:
      str_word = es.getargv(1).lower()
      if str_command == 'ezfilter_add':
         if int_arg_count == 3:
            dict_filtered_words[str_word] = es.getargv(2).lower()
         else:
            dict_filtered_words[str_word] = ''
      elif dict_filtered_words.has_key(str_word):
         del dict_filtered_words[str_word]
   else:
      es.dbgmsg(0, 'Syntax ' + {'ezfilter_add':'ezfilter_add <word> [replacement word]', 'ezfilter_remove':'ezfilter_remove <word>'}[str_command])


def say_command_cmd():
   """
   ezfilter_command_add <command> [0 = do not filter words after command, 1 = filter words after command]
   ezfilter_command_remove <command>
   Adds or removes the say command from dict_say_commands
   """
   global dict_say_commands

   str_command = es.getargv(0).lower()
   int_arg_count = es.getargc()
   if int_arg_count in {'ezfilter_command_add':(2, 3), 'ezfilter_command_remove':(2,)}[str_command]:
      str_say_command = es.getargv(1).lower()
      if str_command == 'ezfilter_command_add':
         if int_arg_count == 3:
            int_filter = int(es.getargv(2))
         else:
            int_filter = 1

         dict_say_commands[str_say_command] = int_filter
      elif dict_say_commands.has_key(str_say_command):
         del dict_say_commands[str_say_command]
   else:
      es.dbgmsg(0, 'Syntax ' + {'ezfilter_command_add':'ezfilter_command_add <command> [0 = do not filter words after command, 1 = filter words after command]', 'ezfilter_command_remove':'ezfilter_command_remove <command>'}[str_command])
   


def immunity_cmd():
   """
   ezfilter_immunity_add <"Steam ID"> [0 = immune from punishments, 1 = immune from punishments and filters]
   ezfilter_immunity_remove <"Steam ID">
   Adds or removes filter immunity from the specified Steam ID
   """
   global dict_players

   str_command = es.getargv(0).lower()
   int_arg_count = es.getargc()
   if int_arg_count in {'ezfilter_immunity_add':(2, 3), 'ezfilter_immunity_remove':(2,)}[str_command]:
      str_steamid = es.getargv(1).upper()
      if str_command == 'ezfilter_immunity_add':
         if not dict_players.has_key(str_steamid):
            dict_players[str_steamid] = dict_player_template.copy()

         if int_arg_count == 3:
            int_immunity = int(es.getargv(2)) + 1
         else:
            int_immunity = 1

         dict_players[str_steamid]['immune'] = int_immunity
      elif dict_players.has_key(str_steamid):
         dict_players[str_steamid]['immune'] = 0
   else:
      es.dbgmsg(0, 'Syntax ' + {'ezfilter_immunity_add':'ezfilter_immunity_add <\"Steam ID\"> [0 = immune from punishments, 1 = immune from punishments and filters]', 'ezfilter_immunity_remove':'ezfilter_immunity_remove <\"Steam ID\">'}[str_command])


def save_strikes():
   """Saves the database of Steam IDs and strikes"""
   str_file_path = es.getAddonPath('ezfilter') + '/ezfilter_strikes.db'
   file_database = open(str_file_path, 'w')
   for str_steamid in dict_players:
      int_strikes = dict_players[str_steamid]['filter_count']
      if int_strikes:
         file_database.write('%s=%s' % (str_steamid, int_strikes))
   file_database.close()


def load_events():
   """Loads the .res file for the events"""
   if os.path.isfile(es.getAddonPath('ezfilter') + '/ezfilter.res'):
      es.loadevents('declare', 'addons/eventscripts/ezfilter/ezfilter.res')
   else:
      if not os.path.isfile(es.getAddonPath('ezfilter') + 'es_ezfilter_events_db.txt'):
         es.keygroupcreate('ezfilter_events')
         for str_event in dict_events:
            es.keycreate('ezfilter_events', str_event)
            for str_event_var in dict_events[str_event]:
               es.keysetvalue('ezfilter_events', str_event, str_event_var, dict_events[str_event][str_event_var])
         es.keygroupsave('ezfilter_events', '|ezfilter')
         es.keygroupdelete('ezfilter_events')
      es.loadevents('declare', 'addons/eventscripts/ezfilter/es_ezfilter_events_db.txt')
      es.dbgmsg(0, 'EZFilter: Unable to load ezfilter.res! Please ensure it is in the ./ezfilter/ directory.')


def remove_tags(str_text):
   """Removes #lightgreen, #green and #default tags from the supplied string"""
   return str_text.replace('#lightgreen', '').replace('#green', '').replace('#default', '')


def missing_lang(str_heading, dict_tokens, str_language):
   """Displays an error message because the languages .ini could not be found"""
   return 'EZFilter: ezfilter_languages.ini not found'